/* FLAC Plugin for Soundplay
    Copyright (C) 2005 Ville Koskinen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "ConfigurationView.h"

#include <stdio.h>

ConfigurationView::ConfigurationView(BMessage *conf):
	BView(BRect(0, 0, 200, 60), "configuration root view",
		B_FOLLOW_ALL_SIDES, B_WILL_DRAW)
{
	int32 current_encoding;
	BRect b;

	this->conf = conf;

	b = Bounds();
	BView *bg = new BView(b, "background", 
		B_FOLLOW_ALL_SIDES, B_WILL_DRAW);
	bg->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	
	AddChild(bg);
	
	b = Bounds();
	b.InsetBy(5, 5);
	b.bottom = b.top + 30;
	assume_utf8 = new BCheckBox(b,
		"assume utf8", "Assume metadata is UTF8", new BMessage(CV_ASSUME_UTF8),
		B_FOLLOW_LEFT_RIGHT);
	
	bg->AddChild(assume_utf8);
	
	BPopUpMenu *ch_codes = new BPopUpMenu("<encoding>", true, true, B_ITEMS_IN_COLUMN);

	static encoding_name encodings[] = {
		{ B_ISO1_CONVERSION,				"ISO 8859-1" },
		{ B_ISO2_CONVERSION,				"ISO 8859-2" },
		{ B_ISO3_CONVERSION,				"ISO 8859-3" },
		{ B_ISO4_CONVERSION,				"ISO 8859-4" },
		{ B_ISO5_CONVERSION,				"ISO 8859-5" },
		{ B_ISO6_CONVERSION,				"ISO 8859-6" },
		{ B_ISO7_CONVERSION,				"ISO 8859-7" },
		{ B_ISO8_CONVERSION,				"ISO 8859-8" },
		{ B_ISO9_CONVERSION,				"ISO 8859-9" },
		{ B_ISO10_CONVERSION,				"ISO 8859-10" },
		{ B_ISO13_CONVERSION,				"ISO 8859-13" },
		{ B_ISO14_CONVERSION,				"ISO 8859-14" },
		{ B_ISO15_CONVERSION,				"ISO 8859-15" },
		{ 0, "-" },
		{ B_MAC_ROMAN_CONVERSION,			"Macintosh Roman" },
		{ 0, "-" },
		{ B_SJIS_CONVERSION,				"Shift-JIS" },
		{ B_EUC_CONVERSION,				"EUC Packed Japanese" },
		{ B_JIS_CONVERSION,				"JIS X 0208-1990" },
		{ B_EUC_KR_CONVERSION,			"EUC Korean" },
		{ B_KOI8R_CONVERSION,				"KOI8-R" },
		{ 0, "-" },
		{ B_MS_WINDOWS_1251_CONVERSION,	"MS-Windows Codepage 1251" },
		{ B_MS_WINDOWS_CONVERSION,		"MS-Windows Codepage 1252" },
		{ B_MS_DOS_CONVERSION,			"MS-DOS Codepage 437" },
		{ B_MS_DOS_866_CONVERSION,		"MS-DOS Codepage 866" },
		{ 0, "-" },
		{ B_UNICODE_CONVERSION,			"Unicode 2.0" },
		{ 0, NULL }
	};
	
	if (conf->FindInt32("encoding", &current_encoding) != B_OK) {
		current_encoding = B_ISO1_CONVERSION;
		conf->AddInt32("encoding", current_encoding);
	}
	
	bool utf8;
	if (conf->FindBool("assume_utf8", &utf8) != B_OK) {
		assume_utf8->SetValue(B_CONTROL_ON);
		conf->AddBool("assume_utf8", true);
	}
	assume_utf8->SetValue(utf8 ? B_CONTROL_ON : B_CONTROL_OFF);
	
	int i = 0;
	while (1) {
		if (encodings[i].name == NULL)
			break;
			
		if (strcmp(encodings[i].name, "-") == 0) {
			ch_codes->AddSeparatorItem();
			i++;
			continue;
		}
	
		BMessage *encoding = new BMessage(CV_ENCODING_SELECT);
		encoding->AddInt32("encoding", encodings[i].type);
		BMenuItem *item = new BMenuItem(encodings[i].name, encoding);
		
		item->SetMarked(false);
		if (encodings[i].type == current_encoding)
			item->SetMarked(true);

		ch_codes->AddItem(item);

		i++;
	}

	b = Bounds();
	b.InsetBy(5, 5);
	b.top = assume_utf8->Bounds().bottom + 5;
	b.bottom = b.top + 30;
	select_encoding = new BMenuField(b,
		"encoding selection", "Metadata encoding:", ch_codes,
		B_FOLLOW_ALL_SIDES, B_WILL_DRAW
		);
	
	if (assume_utf8->Value() == B_CONTROL_ON)
		select_encoding->SetEnabled(false);
	
	bg->AddChild(select_encoding);

	my_message_filter = new ConfigurationMessageFilter(this);
}

ConfigurationView::~ConfigurationView()
{
	delete my_message_filter;
}

void ConfigurationView::AttachedToWindow()
{
	if (Window()) {
		Window()->SetFilterList(NULL);
		Window()->AddFilter(my_message_filter);
		BRect b = Bounds();
		b.OffsetBy(10, 10);
		Window()->ResizeTo(b.right, b.bottom);
	}
	
	BView::AttachedToWindow();
}

void ConfigurationView::DetachedFromWindow()
{
	if (Window()) {
		Window()->RemoveFilter(my_message_filter);
	}
	
	BView::DetachedFromWindow();
}
	
void ConfigurationView::MessageReceived(BMessage *message)
{
	switch (message->what) {
		case CV_ENCODING_SELECT:
			int32 encoding;
			if (message->FindInt32("encoding", &encoding) == B_OK) {
				conf->ReplaceInt32("encoding", encoding);
			}
			break;
			
		case CV_ASSUME_UTF8:
			if (assume_utf8->Value() == B_CONTROL_ON) {
				select_encoding->SetEnabled(false);
				conf->ReplaceBool("assume_utf8", true);
			}
			else {
				select_encoding->SetEnabled(true);
				conf->ReplaceBool("assume_utf8", false);
			}
			break;
			
		default:
			BView::MessageReceived(message);
			break;
	}
}

ConfigurationMessageFilter::ConfigurationMessageFilter(BHandler *target):
	BMessageFilter(B_ANY_DELIVERY, B_ANY_SOURCE)
{
	this->target = target;
}

ConfigurationMessageFilter::~ConfigurationMessageFilter()
{
}

filter_result ConfigurationMessageFilter::Filter(BMessage *message, BHandler **handler)
{
	switch (message->what) {
		case CV_ENCODING_SELECT: // fall through
		case CV_ASSUME_UTF8:
			*handler = target;
			break;
		default:
			break;
	}

	return B_DISPATCH_MESSAGE;
}

	
