/*  FLAC Plugin for Soundplay
    Copyright (C) 2005 Ville Koskinen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <Alert.h>

#include <support/UTF8.h>

/* open, lseek, read, etc. */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "FLACPlugin.h"
#include "ConfigurationView.h"

/****/
/* Exported functions. */
static plugin_descriptor **get_plugin_list(void)
{
	static plugin_descriptor *plugs[] = {
		&fp_descriptor,
		NULL
	};
	
	return plugs;
}

static supported_type *get_supported_types(void)
{
	return fp_supported_types;
}
/****/

/****/
/* Decoder functions. */
static void *fp_instantiate(void **data, const char *name, const char *header, 
	uint32 size, plugin_info *pluginfo, SoundPlayBufferedReader *reader)
{
	if(pluginfo->controller->Version() < 0x00049000)
		return NULL;
	
	DecoderInterface *fd = NULL;
	try {
		switch (DetectContent(name, header, size)) {
			case FLAC_TRANSPORT:
				fd = new FLACDecoder(pluginfo->controller);
				break;
		
			case OGG_TRANSPORT:
				fd = new OggFLACDecoder(pluginfo->controller);
				break;
		
			default:
				return NULL;
				break;
		}
	}
	catch (...) {
		delete fd;
		return NULL;
	}
	
	*data = fd;
	
	if (reader->IsRandomlyAccessible())
		return &fp_file_ops;
	
	return &fp_stream_ops;
}

static void fp_destroy(void *plugin_ops, void *data)
{
	DecoderInterface *fd = (DecoderInterface *)data;
	delete fd;
}

static status_t fp_open(void *data, const char *name, const char *header, 
	uint32 size, BMessage *config, SoundPlayBufferedReader *reader)
{
	DecoderInterface *fd = (DecoderInterface *)data;

	fd->Configure(config);	
	return fd->Open(reader);
}

static void fp_close(void *data)
{
	DecoderInterface *fd = (DecoderInterface *)data;
	fd->Close();
}

static int32 fp_read(void *data, char* buf, ulong count)
{
	DecoderInterface *fd = (DecoderInterface *)data;
	int32 read = fd->Read(buf, count);
	//printf("read %ld bytes\n", read);
	return read;
}

static status_t fp_info(void *data, file_info *info)
{
	DecoderInterface *fd = (DecoderInterface *)data;
	return fd->GetInfo(info);
}

static status_t fp_seek(void *data, uint32 pos)
{
	DecoderInterface *fd = (DecoderInterface *)data;
	return fd->Seek(pos);
}

static uint32 fp_position(void *data)
{
	DecoderInterface *fd = (DecoderInterface *)data;
	uint32 pos = fd->GetPosition();
	return pos;
}

static float fp_bufferamount(void *data)
{
	DecoderInterface *fd = (DecoderInterface *)data;
	return fd->BufferAmount();
}

static void fp_about(void)
{
	BAlert *ba = new BAlert(
		"About FLAC plugin",
		"FLAC plugin for SoundPlay.\n\n"
		"Uses libFLAC++ and libOggFLAC++ from http://flac.sourceforge.net/",
		"OK",
		NULL,
		NULL,
		B_WIDTH_AS_USUAL,
		B_INFO_ALERT);
	ba->Go();
}

static BView* fp_configure(BMessage *message)
{
	ConfigurationView *cv = new ConfigurationView(message);
	return cv;
}

/****/

static TRANSPORT_TYPE DetectContent(const char *name, const char *header, uint32 size)
{
	if (strncmp(header, FLAC_MAGIC_STRING, strlen(FLAC_MAGIC_STRING)) == 0)
		return FLAC_TRANSPORT;
	
	/* http://flac.sourceforge.net/ogg_mapping.html */
	if (strncmp(header, OGGFLAC_MAGIC_STRING, strlen(OGGFLAC_MAGIC_STRING)) == 0) {
		int fd = open(name, O_RDONLY);
		if (fd < 0)
			return NO_TRANSPORT;
		
		/* In the Ogg header, the byte 26 will contain the number of page_segments. */
		lseek(fd, 26, SEEK_SET);
		uint8 page_segments;
		read(fd, &page_segments, 1);
		
		/* Knowing how many segments there are, let's seek until the end of header. */
		lseek(fd, page_segments, SEEK_CUR);
		
		struct flac_packet_header {
			uint8 packet_type;
			uint8 signature[4];
			uint8 version_major;
			uint8 versio_minor;
			uint8 num_of_header_packets[2];
			uint8 native_signature[4];
		} packet_header;
		
		int count = read(fd, &packet_header, sizeof(flac_packet_header));
		
		close(fd);
		
		if (count != sizeof(packet_header)) {
			return NO_TRANSPORT;
		}
		
		if (strncmp((char *)packet_header.native_signature, FLAC_MAGIC_STRING, 4) == 0) {
			return OGG_TRANSPORT;
		}
	}
		
	return NO_TRANSPORT;
}


/****/
/* DecoderInterface */

DecoderInterface::DecoderInterface(SoundPlayController *controller)
	:	fMetadataAssumeUTF8(true),
		fMetadataEncoding(B_ISO1_CONVERSION)
{
	fController = controller;
}

DecoderInterface::~DecoderInterface()
{
}

void DecoderInterface::Configure(BMessage *config)
{
	if (config->FindBool("assume_utf8", &fMetadataAssumeUTF8) != B_OK)
		fMetadataAssumeUTF8 = true;
	if (config->FindInt32("encoding", &fMetadataEncoding) != B_OK)
		fMetadataEncoding = B_ISO1_CONVERSION;
}

/* Decoder */

template <class DecoderImpl>
Decoder<DecoderImpl>::Decoder(SoundPlayController *controller)
	:	DecoderInterface(controller),
		fProcessedMetadata(false)
{
	fDecoder = NULL;
}

template <class DecoderImpl>
Decoder<DecoderImpl>::~Decoder()
{
	if (fDecoder) {
		fDecoder->finish();
		delete fDecoder;
	}
}

template <class DecoderImpl>
status_t Decoder<DecoderImpl>::Open(SoundPlayBufferedReader *reader)
{			
	fDecoder = new DecoderImpl(reader);

	fDecoder->init();
	status_t status = CheckMetadata();
	
	return status;
}

template <class DecoderImpl>
status_t Decoder<DecoderImpl>::CheckMetadata(void)
{
	//if (!fProcessedMetadata) {
		fDecoder->process_until_end_of_metadata();
		
		if (fDecoder->get_state() == FLAC__STREAM_DECODER_ABORTED)
			return B_ERROR;
		
		//if (fDecoder->IsMetadataRead()) {
			BString metadata;
			if (IsMetadataAssumeUTF8()) {
				metadata = fDecoder->GetMetadata();
				printf("Assuming UTF8\n");
			}
			else {
				char dest_string[METADATA_STRING_LENGTH];
				int32 source_length = METADATA_STRING_LENGTH;
				int32 dest_length = METADATA_STRING_LENGTH;
				int32 state = 0;
				
				printf("Converting metadata to UTF8\n");
				
				convert_to_utf8(GetMetadataEncoding(), fDecoder->GetMetadata(),
					&source_length, dest_string, &dest_length, &state);
				
				metadata = dest_string;
			}
			
			GetController()->SetMetaData(metadata.String(), this);
		//}
			
		fProcessedMetadata = true;
		
		return B_OK;
	//}
	
	return B_OK;
}

template <class DecoderImpl>
int32 Decoder<DecoderImpl>::Read(char *buffer, ulong count)
{
	uint32 decoded = 0;

	if (fDecoder->get_state() == FLAC__STREAM_DECODER_END_OF_STREAM) {
		return 0;
	}

	if ( fDecoder->SamplesLeft() < fDecoder->FramesToSamples(count) )
	{
		printf("Trying to read %lu samples and %llu are left\n",
		fDecoder->FramesToSamples(count), fDecoder->SamplesLeft());
		
		if (fDecoder->SamplesLeft() == 0) {
			fDecoder->SeekToEnd();
			return 0;
		}
				
		uint32 decoded_count = 0;

		fDecoder->process_until_end_of_stream();
	
		fDecoder->GetBuffer(buffer, &decoded_count);
		
		decoded += decoded_count;
		buffer += decoded_count;
	}
	else do {
		uint32 decoded_count = 0;

		fDecoder->process_single();

		if (fDecoder->IsAborted())
			/* FIXME: (or test) what happens when the file really is broken and we
			get here? Does SoundPlay stop playing, or...? */
			break;
		
		fDecoder->GetBuffer(buffer, &decoded_count);
		
		decoded += decoded_count;
		buffer += decoded_count;

		if (fDecoder->get_state() == FLAC__STREAM_DECODER_END_OF_STREAM) {
			break;
		}
	} while (fDecoder->SamplesToFrames(decoded) <= count);

	return fDecoder->SamplesToFrames(decoded);
}		

template <class DecoderImpl>
status_t Decoder<DecoderImpl>::GetInfo(file_info *info)
{
	if (fDecoder->IsAborted())
		return B_ERROR;

	*info = fDecoder->GetInfo();
	
	return B_OK;
}

/****/

