/*  FLAC Plugin for Soundplay
    Copyright (C) 2005 Ville Koskinen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/* Partly modeled after the OggVorbis plugin for Soundplay by Marco Nelissen. */

#ifndef _FLACPLUGIN_H_
#define _FLACPLUGIN_H_

#include <Message.h>
#include <View.h>

#include "pluginproto.h"
#include "StreamDecoder.h"

enum TRANSPORT_TYPE {
	NO_TRANSPORT,
	FLAC_TRANSPORT,
	OGG_TRANSPORT
};

/****/
/* SoundPlay stuff. */

static void *fp_instantiate(void **data, const char *name, const char *header, 
		uint32 size, plugin_info *pluginfo, SoundPlayBufferedReader *reader);
static void fp_destroy(void *plugin_ops, void *data);

static void fp_about(void);
static BView* fp_configure(BMessage *message);

static status_t fp_open(void *data,const char *name, const char *header, 
	uint32 size, BMessage *config, SoundPlayBufferedReader *reader);
static void fp_close(void *data);

static int32 fp_read(void *data, char* buf, ulong count);
static status_t fp_info(void *data, file_info *info);
static status_t fp_seek(void *data, uint32 pos);
static uint32 fp_position(void *data);
static float fp_bufferamount(void *data);

static plugin_descriptor fp_descriptor = {
	PLUGIN_DESCRIPTOR_MAGIC,
	PLUGIN_DESCRIPTOR_VERSION,
	
	"sp-flac",
	FP_VERSION,
	PLUGIN_IS_DECODER | PLUGIN_NEEDS_READER,
	
	"FLAC Decoder",
	"By Ville Koskinen.\n\n"
	"Uses libFLAC++ and libOggFLAC++ by the FLAC Project (see\n"
	"http://flac.sourceforge.net)",
	
	fp_about,
	fp_configure,
	
	&fp_instantiate,
	&fp_destroy
};

static supported_type fp_supported_types[] = {
	{ "audio/x-flac", "FLAC audio file", "FLAC audio file", "flac", NULL },
	{ NULL, NULL, NULL, NULL, NULL }
};	

static decoder_plugin_ops fp_file_ops = {
	PLUGIN_DECODER_MAGIC,
	PLUGIN_DECODER_VERSION,
	
	fp_open,
	fp_close,
	fp_info,
	fp_read,
	NULL,	// play
	NULL,	// stop
	NULL,	// setvolume
	NULL,	// setpitchandspeed
	fp_seek,
	fp_position,
	NULL	// bufferamount
};

static decoder_plugin_ops fp_stream_ops = {
	PLUGIN_DECODER_MAGIC,
	PLUGIN_DECODER_VERSION,
	
	fp_open,
	fp_close,
	fp_info,
	fp_read,
	NULL,	// play
	NULL,	// stop
	NULL,	// setvolume
	NULL,	// setpitchandspeed
	NULL, 	// seek,
	NULL, 	// position,
	fp_bufferamount
};

/****/

static TRANSPORT_TYPE DetectContent(const char *name, const char *header, uint32 size);

class DecoderInterface
{
private:
	SoundPlayController *fController;
	
	bool fMetadataAssumeUTF8;
	int32 fMetadataEncoding;

protected:
	bool IsMetadataAssumeUTF8() const { return fMetadataAssumeUTF8; }
	int32 GetMetadataEncoding() const { return fMetadataEncoding; }
	SoundPlayController* GetController() const { return fController; }

public:
	DecoderInterface(SoundPlayController *controller);
	virtual ~DecoderInterface();

	void Configure(BMessage *config);

	virtual status_t Open(SoundPlayBufferedReader *reader) = 0;
	virtual void Close() = 0;
	
	virtual int32 Read(char *buffer, ulong count) = 0;
	virtual status_t GetInfo(file_info *info) = 0;
	
	virtual status_t Seek(uint32 pos) = 0;
	virtual uint32 GetPosition() = 0;
	
	virtual float BufferAmount() = 0;
};

template <class DecoderImpl>
class Decoder: public DecoderInterface
{
private:
	DecoderImpl *fDecoder;

	bool fProcessedMetadata;

public:
	Decoder(SoundPlayController *controller);
	~Decoder();
		
	status_t Open(SoundPlayBufferedReader *reader);
	void Close() { }
	
	int32 Read(char *buffer, ulong count);
	status_t GetInfo(file_info *info);

	status_t Seek(uint32 pos) {
		// seek_absolute() seeks by sample
		return (fDecoder->seek_absolute(pos)) ? B_OK : B_ERROR; 
	}
	
	uint32 GetPosition(void) { return fDecoder->GetPosition(); }
	float BufferAmount(void) { return fDecoder->BufferAmount(); }

private: 
	status_t CheckMetadata();
};

typedef Decoder<FLACSeekableStreamDecoder> FLACDecoder;
typedef Decoder<OggFLACSeekableStreamDecoder> OggFLACDecoder;

#endif // _FLACPLUGIN_H_
