/*  FLAC Plugin for Soundplay
    Copyright (C) 2005 Ville Koskinen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "StreamDecoder.h"

#include <ByteOrder.h> /* B_HOST_IS_LENDIAN etc. */
#include <MediaDefs.h> /* B_LITTLE_ENDIAN etc. */

/****/
GenericSeekableStreamDecoder::GenericSeekableStreamDecoder(
	SoundPlayBufferedReader *reader)
	:	fAbort(false), 
		fMetadataRead(false),
		fPosition(0),
		fTotalSamples(0),
		fSamplesToEndOfFrame(0),
		fLengthInMS(0)
{
	fReader = reader;
 	fAbort = false;
 	fAudioBufferSize = 0; 
 	/* Let's just allocate something small enough so that it gets resized
 	in the write callback. */
 	fAudioBufferMaxSize = 16;
 	fAudioBuffer = (char *)malloc(fAudioBufferMaxSize);
 	fMetadataString[0] = '\0';
 	
 	SetSettings();
 	
 	if (GetReader()->IsRandomlyAccessible())
	 	printf("This stream is randomly accessible.\n");
	//printf("length = %lld\n", GetReader()->Size());
}

GenericSeekableStreamDecoder::~GenericSeekableStreamDecoder()
{
	free(fAudioBuffer);
}

void GenericSeekableStreamDecoder::GetBuffer(char *buffer, uint32 *size) const
{
	memmove(buffer, fAudioBuffer, fAudioBufferSize);
	*size = fAudioBufferSize;
}

/* Callbacks. */

FLAC__SeekableStreamDecoderReadStatus
	GenericSeekableStreamDecoder::read_callback(FLAC__byte buffer[], unsigned *bytes)
{
	int read = GetReader()->Read((void *)buffer, *bytes);
	FLAC__SeekableStreamDecoderReadStatus status;
	if (read <= 0) 
		status = FLAC__SEEKABLE_STREAM_DECODER_READ_STATUS_ERROR;
	else	
		status = FLAC__SEEKABLE_STREAM_DECODER_READ_STATUS_OK;
	
	return status;
}

FLAC__StreamDecoderWriteStatus 
	GenericSeekableStreamDecoder::write_callback(const ::FLAC__Frame *frame,
		const FLAC__int32 * const buffer[])
{
	/* Keep up-to-date information about the current position in samples. We need
	to do this manually, since FLAC__file_decoder_get_decode_position() gives the
	position by _byte_, not by sample. 
	
	Note that this isn't accurate. The frame header contains the first sample in
	the frame, so there's trouble ahead at the end of file. (See FLACDecoder::read())
	*/
	fSamplesToEndOfFrame = frame->header.blocksize * frame->header.channels;

	if (frame->header.number_type == FLAC__FRAME_NUMBER_TYPE_FRAME_NUMBER)
		fPosition = FramesToSamples(frame->header.number.frame_number);
	else
		fPosition = frame->header.number.sample_number;
	
	/* If the buffer's not big enough, make it so. */	
	if (frame->header.blocksize > fAudioBufferMaxSize) {
		fAudioBufferMaxSize = 
			frame->header.blocksize * fFileInfo.numchannels * fFileInfo.samplesize;

		fAudioBuffer = (char *)realloc(fAudioBuffer, fAudioBufferMaxSize);
	}
	
	if (B_HOST_IS_LENDIAN)
		fAudioBufferSize = dither::FLAC__plugin_common__pack_pcm_signed_little_endian(
			(FLAC__byte *)fAudioBuffer, /* is this typecasting safe? */
			buffer, 
			frame->header.blocksize,
			frame->header.channels,
			frame->header.bits_per_sample, 
			16
		);
	else 
		fAudioBufferSize = dither::FLAC__plugin_common__pack_pcm_signed_big_endian(
			(FLAC__byte *)fAudioBuffer, 
			buffer, 
			frame->header.blocksize,
			frame->header.channels, 
			frame->header.bits_per_sample,
			16 
		);
	
	return FLAC__STREAM_DECODER_WRITE_STATUS_CONTINUE;
}

void GenericSeekableStreamDecoder::metadata_callback(
	const ::FLAC__StreamMetadata *metadata)
{
	if (metadata->type == FLAC__METADATA_TYPE_STREAMINFO) {
		fFileInfo.samplerate = metadata->data.stream_info.sample_rate;
		fFileInfo.numchannels = metadata->data.stream_info.channels;
		fFileInfo.samplesize = metadata->data.stream_info.bits_per_sample / 8;
		fFileInfo.granularity = 1;
		
		if (!GetReader()->IsRandomlyAccessible()) {
			fFileInfo.framecount = uint32(-1);
			fTotalSamples = uint32(0x7fffffff);
			fLengthInMS = uint32(0x7fffffff);
			fFileInfo.bitrate = fFileInfo.samplesize * 8 * fFileInfo.samplerate;
		}
		else {
			fTotalSamples = metadata->data.stream_info.total_samples * fFileInfo.numchannels * fFileInfo.samplesize;
			fLengthInMS = (unsigned)((double)fTotalSamples 
				/ (double)fFileInfo.samplerate * 1000.0 + 0.5);
			fFileInfo.framecount = 
				SamplesToFrames(fTotalSamples); 
			fFileInfo.bitrate = fTotalSamples * fFileInfo.samplesize * 8 
				/ fLengthInMS * 1000;
		}
		
		printf("Samples = %llu\n", fTotalSamples);
		printf("Frames = %ld\n", fFileInfo.framecount);
		
		if (B_HOST_IS_LENDIAN)
			fFileInfo.byteorder = B_LITTLE_ENDIAN;
		else
			fFileInfo.byteorder = B_BIG_ENDIAN;
		
		fFileInfo.sampleformat = B_LINEAR_SAMPLES;
		
		fFileInfo.flags = 0;
		
		if(!GetReader()->IsRandomlyAccessible())
			fFileInfo.flags |= PLUGIN_FILELENGTH_UNKNOWN;
		
		strcpy(fFileInfo.name, "");
		
		char channels[30];
		if (fFileInfo.numchannels == 1)
			strcpy(channels, "mono");
		else if (fFileInfo.numchannels == 2)
			strcpy(channels, "stereo");
		else
			// SoundPlay only supports mono or stereo, but never mind about that.
			snprintf(channels, 30, "%ld channels", fFileInfo.numchannels);
		
		snprintf(fFileInfo.typedesc, PLUGIN_STRING_LENGTH, "FLAC %g kHz, %ld-bit %s",
			fFileInfo.samplerate / 1000,
			fFileInfo.samplesize * 8,
			channels
			);
			
		strcpy(fFileInfo.mimetype, "audio/x-flac");
	}
	else if (metadata->type == FLAC__METADATA_TYPE_VORBIS_COMMENT) {
		char artist[METADATA_COMMENT_LENGTH];
		char title[METADATA_COMMENT_LENGTH];
		const FLAC__StreamMetadata_VorbisComment *vc = &metadata->data.vorbis_comment;
		
		for (uint32 i = 0; i < vc->num_comments; i++) {
			if (strncasecmp("artist=", (char *)vc->comments[i].entry, 7) == 0)
				strncpy(artist, (char *)(vc->comments[i].entry+7), 
					METADATA_COMMENT_LENGTH);
			else if (strncasecmp("title=", (char *)vc->comments[i].entry, 6) == 0) 
				strncpy(title, (char *)(vc->comments[i].entry+6), 
					METADATA_COMMENT_LENGTH);
		}
		
		if (strcmp(artist, "") != 0 && strcmp(title, "") != 0)
			snprintf(fMetadataString, METADATA_STRING_LENGTH,
				METADATA_TEMPLATE_FULL, artist, title);
		else if (strcmp(title, "") != 0)
			snprintf(fMetadataString, METADATA_STRING_LENGTH,
				METADATA_TEMPLATE_TITLE, title);
		
	}

	fMetadataRead = true;
}

void GenericSeekableStreamDecoder::error_callback(
	::FLAC__StreamDecoderErrorStatus status)
{
	if (status != FLAC__STREAM_DECODER_ERROR_STATUS_LOST_SYNC)
		fAbort = true;
}

FLAC__SeekableStreamDecoderSeekStatus 
	GenericSeekableStreamDecoder::seek_callback(FLAC__uint64 absolute_byte_offset)
{
	if (!GetReader()->IsRandomlyAccessible())
		return FLAC__SEEKABLE_STREAM_DECODER_SEEK_STATUS_ERROR;
	
	FLAC__uint64 pos = (FLAC__uint64)GetReader()->Seek(absolute_byte_offset, SEEK_SET);
	FLAC__SeekableStreamDecoderSeekStatus status = 
		(pos != absolute_byte_offset)
			? FLAC__SEEKABLE_STREAM_DECODER_SEEK_STATUS_ERROR
			: FLAC__SEEKABLE_STREAM_DECODER_SEEK_STATUS_OK;
	
	return status;
}

FLAC__SeekableStreamDecoderTellStatus 
	GenericSeekableStreamDecoder::tell_callback(FLAC__uint64 *absolute_byte_offset)
{
	if (!GetReader()->IsRandomlyAccessible())
		return FLAC__SEEKABLE_STREAM_DECODER_TELL_STATUS_ERROR;

	*absolute_byte_offset = (FLAC__uint64)GetReader()->Position();

	return FLAC__SEEKABLE_STREAM_DECODER_TELL_STATUS_OK;
}
		
FLAC__SeekableStreamDecoderLengthStatus 
	GenericSeekableStreamDecoder::length_callback(FLAC__uint64 *stream_length)
{
	if (!GetReader()->IsRandomlyAccessible()) {
		*stream_length = long(0x7fffffff);
		return FLAC__SEEKABLE_STREAM_DECODER_LENGTH_STATUS_ERROR;
		//return FLAC__SEEKABLE_STREAM_DECODER_LENGTH_STATUS_OK;
	}

	*stream_length = (FLAC__uint64)GetReader()->Size();
	//*stream_length = (FLAC__uint64)(fTotalSamples * fFileInfo.samplesize * fFileInfo.numchannels);

	return FLAC__SEEKABLE_STREAM_DECODER_LENGTH_STATUS_OK;
}

bool GenericSeekableStreamDecoder::eof_callback()
{
	if (!GetReader()->IsRandomlyAccessible())
		return false;
	
	if (GetReader()->Position() == GetReader()->Size())
		printf("We're at end!\n");
	//return (fPosition == GetReader()->Size() / fFileInfo.numchannels);
	return (GetReader()->Position() == GetReader()->Size());
	//return SamplesLeft() == 0;
}

/****/

