/*  FLAC Plugin for Soundplay
    Copyright (C) 2005 Ville Koskinen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef _STREAMDECODER_H_
#define _STREAMDECODER_H_

#define FLACPP_API		/* a kludge to get things to compile on BeIDE */
#include "FLAC++/decoder.h"
#undef FLACPP_API

#define OggFLACPP_API
#define OggFLAC_API 	/* ditto */
#include "OggFLAC++/decoder.h"
#undef OggFLAC_API
#undef OggFLACPP_API

#include "pluginproto.h"

namespace dither {
	extern "C" {
#include "dither.h"
	}
} // namespace dither


const char * const FLAC_MAGIC_STRING = "fLaC";
const char * const OGGFLAC_MAGIC_STRING = "OggS";
const int FP_VERSION = 1; /* FP as in FLAC Plugin... */

const int METADATA_COMMENT_LENGTH = 128;
const int METADATA_STRING_LENGTH = 256;
const char * const METADATA_TEMPLATE_FULL = "StreamTitle='%s - %s';";
const char * const METADATA_TEMPLATE_TITLE = "StreamTitle='%s';";

class GenericSeekableStreamDecoder
{
protected:
	/* Callbacks */

	FLAC__SeekableStreamDecoderReadStatus read_callback(FLAC__byte buffer[], 
		unsigned *bytes);
	FLAC__StreamDecoderWriteStatus write_callback(const ::FLAC__Frame *frame,
		const FLAC__int32 * const buffer[]);

	void metadata_callback(const ::FLAC__StreamMetadata *metadata);
	void error_callback(::FLAC__StreamDecoderErrorStatus status);
	
	FLAC__SeekableStreamDecoderSeekStatus seek_callback(
		FLAC__uint64 absolute_byte_offset);
	FLAC__SeekableStreamDecoderTellStatus tell_callback(
		FLAC__uint64 *absolute_byte_offset);
	FLAC__SeekableStreamDecoderLengthStatus length_callback(
		FLAC__uint64 *stream_length);
	bool eof_callback();
	
	SoundPlayBufferedReader* GetReader() const { return fReader; }
	
	virtual void SetSettings() { }

private:	
	file_info fFileInfo;
	char fMetadataString[METADATA_STRING_LENGTH];
	
	uint32 fAudioBufferMaxSize;
	char *fAudioBuffer;
	uint32 fAudioBufferSize;
	
	bool fAbort, fMetadataRead;
	
	FLAC__uint64 fPosition;
	
	FLAC__uint64 fTotalSamples;
	FLAC__uint64 fSamplesToEndOfFrame;
	double fLengthInMS;
	
	SoundPlayBufferedReader *fReader;
	
public:
	GenericSeekableStreamDecoder(SoundPlayBufferedReader *reader);
	virtual ~GenericSeekableStreamDecoder();

	/* Access */	
	void GetBuffer(char *buffer, uint32*) const;

	file_info GetInfo(void) const { return fFileInfo; }
	
	FLAC__uint64 GetPosition(void) const { 
		if (!GetReader()->IsRandomlyAccessible())
			return (FLAC__uint64)-1;
		return fPosition; 
	}

	const char* GetMetadata(void) const { return fMetadataString; }

	void SeekToEnd(void) { 
		if (!GetReader()->IsRandomlyAccessible())
			return;
		fPosition = fTotalSamples; //seek_absolute(fPosition); 
	}

	/* Tests */	
	bool IsAborted(void) const { return fAbort; }
	bool IsMetadataRead(void) const { return fMetadataRead; }
	
	/* Utility */
	/* This frame doesn't mean the FLAC type of frame, but SoundPlay type. */
	uint32 SamplesToFrames(uint32 samples) const
	{ return samples / (fFileInfo.numchannels * fFileInfo.samplesize); }
	/*{ uint32 ret = samples / (fFileInfo.numchannels * fFileInfo.samplesize); 
		printf("%ld => SamplesToFrames => %ld\n", samples, ret);
		return ret; }*/
	
	uint32 FramesToSamples(uint32 frames) const
	{ return frames * fFileInfo.numchannels * fFileInfo.samplesize; }
	/*{ uint32 ret = frames * fFileInfo.numchannels * fFileInfo.samplesize; 
		printf("%ld => FramesToSamples => %ld\n", frames, ret);
		return ret; }*/
	
	FLAC__uint64 SamplesLeft(void) const { 
		if (!GetReader()->IsRandomlyAccessible())
			return (uint32)-1;
		// FIXME: Find out what really happens here... see 
		// GenericSeekableStreamDecoder::metadata_callback()
		if (fPosition + fSamplesToEndOfFrame >= fTotalSamples/fFileInfo.numchannels/fFileInfo.samplesize)
			return 0;
		FLAC__uint64 left = fTotalSamples/fFileInfo.numchannels/fFileInfo.samplesize - fPosition; 
		//printf("Samples left = %llu\n", left);
		return left;
	}
	
	float BufferAmount(void) const { return GetReader()->BufferPercentage(); }
};

class FLACSeekableStreamDecoder: 
	public FLAC::Decoder::SeekableStream,
	public GenericSeekableStreamDecoder
{
protected:
	FLAC__SeekableStreamDecoderReadStatus read_callback(FLAC__byte buffer[], 
		unsigned *bytes) 
	{ return GenericSeekableStreamDecoder::read_callback(buffer, bytes); }
	FLAC__StreamDecoderWriteStatus write_callback(const ::FLAC__Frame *frame,
		const FLAC__int32 * const buffer[]) 
	{ return GenericSeekableStreamDecoder::write_callback(frame, buffer); }

	void metadata_callback(const ::FLAC__StreamMetadata *metadata)
	{ GenericSeekableStreamDecoder::metadata_callback(metadata); }
	void error_callback(::FLAC__StreamDecoderErrorStatus status)
	{ GenericSeekableStreamDecoder::error_callback(status); }

	FLAC__SeekableStreamDecoderSeekStatus seek_callback(
		FLAC__uint64 absolute_byte_offset)
	{ return GenericSeekableStreamDecoder::seek_callback(absolute_byte_offset); }
	FLAC__SeekableStreamDecoderTellStatus tell_callback(
		FLAC__uint64 *absolute_byte_offset)
	{ return GenericSeekableStreamDecoder::tell_callback(absolute_byte_offset); }
	FLAC__SeekableStreamDecoderLengthStatus length_callback(
		FLAC__uint64 *stream_length)
	{ return GenericSeekableStreamDecoder::length_callback(stream_length); }
	bool eof_callback()
	{ return GenericSeekableStreamDecoder::eof_callback(); }

	virtual void SetSettings() {
		set_metadata_ignore_all();
		set_metadata_respond(FLAC__METADATA_TYPE_STREAMINFO);
		set_metadata_respond(FLAC__METADATA_TYPE_VORBIS_COMMENT);
	}

public:
	FLACSeekableStreamDecoder(SoundPlayBufferedReader *reader):
		GenericSeekableStreamDecoder(reader) { }
	virtual ~FLACSeekableStreamDecoder() { }	
};

class OggFLACSeekableStreamDecoder:
	public OggFLAC::Decoder::SeekableStream,
	public GenericSeekableStreamDecoder
{
protected:
	OggFLAC__SeekableStreamDecoderReadStatus read_callback(FLAC__byte buffer[], 
		unsigned *bytes) 
	{ return (OggFLAC__SeekableStreamDecoderReadStatus)
		GenericSeekableStreamDecoder::read_callback(buffer, bytes); }
	FLAC__StreamDecoderWriteStatus write_callback(const ::FLAC__Frame *frame,
		const FLAC__int32 * const buffer[]) 
	{ return GenericSeekableStreamDecoder::write_callback(frame, buffer); }

	void metadata_callback(const ::FLAC__StreamMetadata *metadata)
	{ GenericSeekableStreamDecoder::metadata_callback(metadata); }
	void error_callback(::FLAC__StreamDecoderErrorStatus status)
	{ GenericSeekableStreamDecoder::error_callback(status); }

	OggFLAC__SeekableStreamDecoderSeekStatus seek_callback(
		FLAC__uint64 absolute_byte_offset)
	{ return (OggFLAC__SeekableStreamDecoderSeekStatus)
		GenericSeekableStreamDecoder::seek_callback(absolute_byte_offset); }
	OggFLAC__SeekableStreamDecoderTellStatus tell_callback(
		FLAC__uint64 *absolute_byte_offset)
	{ return (OggFLAC__SeekableStreamDecoderTellStatus)
		GenericSeekableStreamDecoder::tell_callback(absolute_byte_offset); }
	OggFLAC__SeekableStreamDecoderLengthStatus length_callback(
		FLAC__uint64 *stream_length)
	{ return (OggFLAC__SeekableStreamDecoderLengthStatus)
		GenericSeekableStreamDecoder::length_callback(stream_length); }
	bool eof_callback()
	{ return GenericSeekableStreamDecoder::eof_callback(); }

	virtual void SetSettings() {
		set_metadata_ignore_all();
		set_metadata_respond(FLAC__METADATA_TYPE_STREAMINFO);
		set_metadata_respond(FLAC__METADATA_TYPE_VORBIS_COMMENT);
	}

public:
	OggFLACSeekableStreamDecoder(SoundPlayBufferedReader *reader):
		GenericSeekableStreamDecoder(reader) { }
	virtual ~OggFLACSeekableStreamDecoder() { }
};

#endif // _STREAMDECODER_H_

